Labella.py Documentation
========================

The main interface through which you can use labella.py is through the 
`TimelineSVG` and `TimelineTex` objects in `labella.timeline`. These objects 
are inherit from the main `Timeline` object, which organizes the common code 
needed for both output formats. Both the `TimelineSVG` and `TimelineTex` 
objects support the options implemented in 
[d3kit-timeline](https://github.com/kristw/d3kit-timeline), see the 
documentation 
[here](https://github.com/kristw/d3kit-timeline/blob/master/docs/api.md). Note 
that the options that end in `Fn` require functions, which can be supplied by 
lambdas or user defined functions in Python.

Options that are new and not part of 
[d3kit-timeline](https://github.com/kristw/d3kit-timeline) are:

| option | default | description |
| ------ | ------- | ----------- |
| textXOffset | 0.15em | horizontal offset for text within a label, used for SVG output |
| showTicks | True | boolean for showing ticks on the axis |
| showBorder | False | boolean for showing border around a label |
| borderColor | #000 | color of the border around a label |
| latex | (see below) | options for LaTeX, see below. |

Options for LaTeX are:

| option | default | description |
| ------ | ------- | ----------- |
| fontsize | '11pt' | fontsize of the text in LaTeX document |
| axisThickness | 'very thick' | thickness of the axis in TikZ language |
| borderThickness | 'very thick' | thickness of the border around a label in Tikz language |
| tickThickness | 'thick' | tickness of the tick lines in TikZ language |
| linkThickness | 'very thick' | thickness of the link linkes in TikZ language |
| tickCross | False | boolean for ticks to cross the axis or stay on one side |
| preamble | '' | raw (!) string for the preamble of generated LaTeX documents |
| latexmkOptions | [] | list of additional [arguments](https://man.cx/latexmk#heading4) to pass to latexmk |

Input Specification
-------------------

Labella.py allows input specification through a list of dicts, just like in 
labella.js. A dict must have a 'time' field, which can be either numeric 
(int/float), `datetime.date`, `datetime.time` or `datetime.datetime`. If a 
'width' field is not specified, the default width of 50 will be used. Labels 
with text can be created by supplying a 'text' field in the dict. If this is 
the case and no 'width' field is specified, the width is calculated 
automatically (see Width Calculation below). Other data can be incorporated in 
the dict, which can for instance be used by the `textFn` function, and those 
for determining colors.

Note that if a `datetime.date` or a `datetime.time` object is specified they 
will be converted internally to a full `datetime.datetime` object for use in 
the time scale.

Color Specification
-------------------

As in labella.js, labella.py allows specification of colors through the 
following options to a TimelineSVG or TimelineTex object:

- dotColor
- labelBgColor
- labelTextColor
- linkColor
- borderColor

For these options the color can be given as a three letter hex code (e.g. 
`#222`), a six letter hex code (e.g. `#45f23a`), a user-defined function, a 
lambda expression, or by a list of colors. The default color schemes that are 
implemented are available in `labella.utils` and are known as `COLOR_10` and 
`COLOR_20`, corresponding to the category 10 and category 20 [categorical 
colors in d3](https://github.com/mbostock/d3/wiki/Ordinal-Scales). These are 
defined as lists of 10 and 20 colors respectively. When lists of colors are 
used, the colors are used in order from small time value to large time value, 
looping around when necessary.

Scales
------

Labella.py includes two scales, `LinearScale` and `TimeScale`, which are 
adapted from their d3 equivalents. For use in a Labella timeline their use is 
equivalent to their use in d3kit-timeline, but they have some limitations 
beyond that. For instance, only linear interpolation is implemented, and the 
rangeRound functions of the scales in d3 are not implemented. Additionally, no 
distinction is made between local times and UTC times (i.e. timezones are 
ignored).  This may be a limitation for some, and pull requests for this 
feature are welcome.

PDF output
----------

PDF output can be generated by using the `export` method on a `TimelineTex` 
object. The filename supplied should end in '.tex' since a LaTeX file is 
generated by the export function. Additionally, the `export` method accepts a 
`build_pdf` flag which defaults to `True` and compiles the generated TeX file 
to a PDF with the base filename but with a `.pdf` extension.

Width Calculation
-----------------

In [d3kit-timeline](https://github.com/kristw/d3kit-timeline), the width of a 
label with text is calculated by getting the bounding box of a the HTML label. 
Since labella.py doesn't have access to this, the width is calculated by 
compiling a small LaTeX document which contains only the text, and the 
bounding box is calculated from that. This ensures that for PDF output the 
label has exactly the right width. For the SVG output the width is 
approximate, since a different font is used for SVG than used in LaTeX.

Tests
-----

Labella.py includes unit tests for the d3 scales, VPSC functions, and those 
implemented in labella.js.

Known limitations
-----------------

Margins and initialHeight/initialWidth for the PDF output don't work as well 
as for the SVG output. This may be a problem for some, but in general a tight 
bounding box for a figure should be preferred for incorporating a PDF image in 
another LaTeX file, as it facilitates precise placement.

Labella.py has not been tested on any other platforms than Linux. Therefore, 
the functionality on these platforms may be a bit limited. Pull requests and 
bug reports are welcome.
